/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package com.adventnet.jmx.utils;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.management.ObjectName;

/**
 * The internal utility class used by the jmx extension package classes.
 * This contains the pattern matching static methods useful for ObjectName
 * pattern matching.
 */
public class JmxUtilities
{
	public JmxUtilities()
	{
	}

	public static boolean followsObjectNamePattern(String string, String pattern)
													throws Exception
	{
		ObjectName patON = new ObjectName("tempDomain:"+pattern);
		if(patON.isPropertyPattern())
			return true;

		return false;
	}

	public static boolean objectNameSpecificChecks(String string, String pattern) throws Exception
	{
		ObjectName stringON = new ObjectName("tempDomain:"+string);
		ObjectName patON = new ObjectName("tempDomain:"+pattern);

		Hashtable patProps = patON.getKeyPropertyList();
		Hashtable stringProps = stringON.getKeyPropertyList();

		for(Enumeration e=patProps.keys();e.hasMoreElements();)
		{
			String key = (String)e.nextElement();
			String value = (String)patProps.get(key);

			if(stringProps.get(key) != null && stringProps.get(key).equals(value));
			else
				return false;

		}
		return true;

	}

	public static boolean checkPattern( String info, String pat,boolean wc)
	{
		boolean  returnFlag;

		if(!wc)
		{
			return info.equals(pat);
		}

		try
		{
			if(followsObjectNamePattern(info, pat))
				return objectNameSpecificChecks( info, pat);
		}
		catch(Exception e){}

		returnFlag = wildcardMatch(pat,info);

		return returnFlag;
	}

	public static boolean wildCPattern(String p_string,String p_pattern)
	{
		char  c;
		char  p;
		boolean retval;

		for (;;)
		{
			if(p_pattern.length() == 0) /* End of pattern. If end of string return true */
			{
				if(p_string.length() == 0)
				{
					return true;
				}
				else
				{
					return false;
				}
			}

			if(p_string.length() == 0)
			{
				/*  Critical  condition */
				return false;
			}

			p = p_pattern.charAt(0);
			p_pattern = p_pattern.substring(1);

			if(p == '$')
			{
				if(p_pattern.length() == 0)
				{
					/* Critical condition */
					return false;
				}

				p = p_pattern.charAt(0);
				p_pattern =p_pattern.substring(1);
				//System.out.println("P before switch is " + p);

				switch (p)
				{
				case '*':
					while (p_string.length() != 0)
					{
						p_string = p_string.substring(1);
						//System.out.println("PString is " + p_string);

						/* Match zero or more char */
						retval = wildCPattern(p_string,p_pattern);

						if (retval == true)
						{
							//System.out.println("Returning as retVal true\n");
							return true;
						}
					}

					retval = wildCPattern (p_string,p_pattern);

					return retval;

				case '?':
					/* match any one char */
					if (p_string.length() == 0)
					{
						/* not end of string */
						//System.out.println("Returning as CRITICAL condition III\n");
						return false;
					}
					p_string = p_string.substring(1);

					break;
				case '$':  /* check for $ character */
					p_string = p_string.substring(1);
					c = p_string.charAt(0);
					//System.out.println("C is " + c);
					if (c != '$')
					{
						return false;
					}
					break;
				default:  /* Wild Character Not Found */
					//System.out.println("Returning as NO_WILD CARD character found\n");
					return false;
				} /* End of switch */
			}
			else
			{
				c = p_string.charAt(0);
				p_string = p_string.substring(1);
				if (c != p)            /* check for exact char */
				{
					//System.out.println("Returning as not match condition %c "+c + "\n");
					return false;                   /* not a match */
				}
			}

		} /* End of For */

	}

	public static boolean wildcardMatch(String pattern, String string)
	{
		int stringLength = string.length();
		int stringIndex = 0;
		for (int patternIndex = 0; patternIndex < pattern.length(); ++patternIndex)
		{
			char c = pattern.charAt(patternIndex);
			if (c == '*')
			{
				// Recurse with the pattern without this '*' and the actual string, until
				// match is found or we inspected the whole string
				while (stringIndex < stringLength)
				{
					if (wildcardMatch(pattern.substring(patternIndex + 1), string.substring(stringIndex))) {return true;}
					// No match found, try a shorter string, since we are matching '*'
					++stringIndex;
				}
			}
			else if (c == '?')
			{
				// Increment the string index, since '?' match a single char in the string
                ++stringIndex;
				if (stringIndex > stringLength) {return false;}
			}
			else
			{
				// A normal character in the pattern, must match the one in the string
				if (stringIndex >= stringLength || c != string.charAt(stringIndex)) {return false;}
				++stringIndex;
			}
		}

		// I've inspected the whole pattern, but not the whole string
		return stringIndex == stringLength;
	}
}