/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;

/**
 * Represents an MBean attribute by associating its name with its value. The
 * MBean server and other objects use this class to get and set attributes values.
 */
public class Attribute implements Serializable
{
    /**
     * @serial Attribute name.
     */
	private String name = null;

    /**
     * @serial Attribute value
     */
	private Object value = null;

    /**
     * Constructs an Attribute object which associates the given attribute
	 * name with the given value.
     *
     * @param name A String containing the name of the attribute to be created.
	 * 				Cannot be null.
	 *
     * @param value The Object which is assigned to the attribute. This
	 * 				object must be of the same type as the attribute.
     */
	public Attribute(String name, Object value)
	{
		if(name == null)
		{
		    throw new RuntimeOperationsException(
				new IllegalArgumentException("Attribute name cannot be null "));
		}

		this.name = name;
		this.value = value;
	}

	/**
     * Returns a String containing the  name of the attribute.
     *
	 * @return name of the attribute
	 */
	public String getName()
	{
		return name;
	}

	/**
     * Returns an Object that is the value of this attribute.
     * @return value of the attribute as Object
     */
	public Object getValue()
	{
		return value;
	}

    /**
     * Compares the current Attribute Object with another Attribute Object.
     *
     * @param object  The Attribute that the current Attribute is to be compared with.
     *
     * @return  True if the two Attribute objects are equal, otherwise false.
     */
	public boolean equals(Object obj)
	{
		if(!(obj instanceof Attribute))
			return false;

		Attribute att = (Attribute)obj;
		if(att.getName().equals(name) && att.getValue().equals(value))
			return true;

		return false;
	}
}