/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * This can be implemented by an MBean in order to carry out operations before
 * and after being registered or de-registered from the MBean server.
 */
public abstract interface MBeanRegistration
{
    /**
     * Allows the MBean to perform any operations it needs before being
	 * registered in the MBean server. If the name of the MBean is not
	 * specified, the MBean can provide a name for its registration. If any
	 * exception is raised, the MBean will not be registered in the MBean server.
     *
     * @param server The MBean server in which the MBean will be registered.
     *
     * @param name The object name of the MBean.
     *
     * @return  The name of the MBean registered.
     *
     * @exception Exception This exception should be caught by the MBean server
	 * 				and re-thrown as an <CODE>MBeanRegistrationException</CODE>.
     */
	public ObjectName preRegister(MBeanServer server, ObjectName name)
						throws Exception;

    /**
     * Allows the MBean to perform any operations needed after having been
     * registered in the MBean server or after the registration has failed.
     *
     * @param registrationDone Indicates whether or not the MBean has been
	 * 				successfully registered in the MBean server. The value
	 * 				false means that the registration phase has failed.
     */
	public void postRegister(Boolean registrationDone);

    /**
     * Allows the MBean to perform any operations it needs before being
	 * de-registered by the MBean server.
     *
     * @exception Exception  This exception should be caught by the MBean server
	 * 				and re-thrown as an <CODE>MBeanRegistrationException</CODE>.
     */
	public void preDeregister() throws Exception;

	/**
	 * Allows the MBean to perform any operations needed after having been
	 * de-registered in the MBeanServer.
	 */
	public void postDeregister();
}