/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * When a <CODE>java.lang.Error</CODE> occurs in the agent it should be caught
 * and re-thrown as a <CODE>RuntimeErrorException</CODE>.
 */
public class RuntimeErrorException extends JMRuntimeException
{
    /**
     * Th encapsulated Error
     */
    private Error error = null;

	/**
	 *  Constructs a <CODE>RuntimeErrorException</CODE> wrapping the Error object
	 *
	 *  @param e Error object to be wrapped by <CODE>RuntimeErrorException</CODE>
	 */
	public RuntimeErrorException(Error e)
	{
		this.error = e;
	}

	/**
	 * Constructor that allows a specific error message to be specified.
	 *
	 * @param e Error object to be wrapped by RuntimeErrorException
	 *
	 * @param message The specific error message specified
	 */
	public RuntimeErrorException(Error e, String message)
	{
		super(message);
		this.error = e;
	}

    /**
     * Returns the actual <CODE>java.lang.Error</CODE> thrown.
     *
	 * @return This returns java.lang.Error thrown in this method
     */
    public Error getTargetError()
	{
		return error ;
    }
}//End of class RuntimeErrorException