/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * This class represents runtime exceptions thrown in the agent when
 * performing operations on MBeans.  It wraps the actual
 * <CODE>java.lang.RuntimeException</CODE> thrown.
 */
public class RuntimeOperationsException extends JMRuntimeException
{
    /**
     * The encapsulated RuntimeException
     */
    private RuntimeException runtimeException = null;

	/**
     * Creates a <CODE>RuntimeOperationsException</CODE> that wraps the
	 * actual <CODE>java.lang.RuntimeException</CODE>.
	 *
	 * @param e RuntimeException to be wraped by RuntimeOperationsException
	 */
	public RuntimeOperationsException(RuntimeException e)
	{
		this.runtimeException = e;
	}

	/**
     * Creates a <CODE>RuntimeOperationsException</CODE> that wraps the
	 * actual <CODE>java.lang.RuntimeException</CODE> with a detailed message.
	 *
	 * @param e  RuntimeException to be wraped by RuntimeOperationsException.
	 *
	 * @param message the detail message.
	 */
	public RuntimeOperationsException(RuntimeException e, String message)
	{
		super(message);
		this.runtimeException = e;
	}

	/**
	 * Returns the actual java.lang.RuntimeException thrown
	 *
	 * @return This returns RuntimeException thrown in this method
	 */
	public RuntimeException getTargetException()
	{
		return runtimeException;
	}

	/**
	 * Overrides the method
	 */
	public void printStackTrace()
	{
		super.printStackTrace();

		if(runtimeException != null)
			runtimeException.printStackTrace();
	}

	/**
	 * Overrides the method
	 */
	public String toString()
	{
		if(runtimeException == null || runtimeException.getMessage() == null)
			return super.toString();

		return (super.toString() + "; nested exception is: " +
										runtimeException.toString());
	}
}//End of class RuntimeOperationsException