/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.io.PrintStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.management.ObjectName;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeChangeNotification;
import javax.management.AttributeChangeNotificationFilter;
import javax.management.Descriptor;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.MBeanException;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.ReflectionException;
import javax.management.RuntimeOperationsException;
import javax.management.ServiceNotFoundException;

import javax.jmx.openmbean.CompositeData;
import javax.jmx.openmbean.TabularData;
import javax.jmx.openmbean.TabularParameterInfo;

import com.adventnet.agent.utilities.scheduler.Scheduler;
import com.adventnet.agent.logging.Log;
import com.adventnet.agent.logging.LogFactory;

/**
 * This class is the implementation of a ModelMBean. An appropriate
 * implementation of a ModelMBean must be shipped with every JMX Agent and
 * the class must be named RequiredModelMBean.
 * <p>
 * Java resources wishing to be manageable instatiate the RequiredModelMBean
 * using the MBeanServer's createMBean method. The resource then sets the
 * MBeanInfo and Descriptors for the RequiredModelMBean instance. The
 * attributes and operations exposed via the ModelMBeanInfo for the ModelMBean
 * are accessible from Mbeans, connectors/adapters like other MBeans. Through
 * the Descriptors, values and methods in the managed application can be
 * defined and mapped to attributes and operations of the ModelMBean. This
 * mapping can be defined in an XML formatted file or dynamically and
 * programmatically at runtime.
 * <p>
 * Every RequiredModelMBean which is instantiated in the MBeanServer becomes
 * manageable: its attributes and operations become remotely accessible through
 * the connectors/adaptors connected to that MBeanServer. A Java object cannot
 * be registered in the MBeanServer unless it is a JMX compliant MBean. By
 * instantiating a RequiredModelMBean, resources are guaranteed that the MBean
 * is valid. MBeanException and RuntimeOperatiosException must be thrown on
 * every public method. This allows for wrappering exceptions from distributed
 * communications (RMI, EJB, etc.)
 **/
public class RequiredModelMBean implements ModelMBean
{
	/** The mbean information of this mbean **/
	ModelMBeanInfo mbeanInfo = null;

	/** The actual instrumentaton object. **/
	Object object = null;

	/** The actual instrumentaton object type. **/
	String objectType = null;

	/** The Class object of the instrumentation **/
	Class clazz = null;

	/** The object name of the mbean **/
	private ObjectName objectName = null;

	private Log log;

	private static final String OBJECT_REFERENCE = "ObjectReference";
	private static final String HANDLE = "Handle";
	private static final String IOR = "IOR";
	private static final String EJB_HANDLE = "EJBHandle";
	private static final String RMI_REFERENCE = "RMIReference";

	private final static String direcName = "."+File.separator+"jmxstore";

	/**
	 * The configuration file name which has the list of persistent
	 * information of mbeans.
	 */
	String confFileName = "JMXObjects.conf";

	/** The directory specifying where the conf file resides **/
	String confDirName = direcName;

	/** The complete path ...**/
	String confFile = confDirName + File.separator + confFileName;

	private String storageDirName = null;
	private String storageFileName = null;

	private String persistLocation = null;
	private String persistName     = null;

	private Vector nameVec  =  null;
	private Vector valueVec =  null;

	/** The enable storage flag **/
	private boolean enableStorage = true;

	/** indicated whether the storage is allowed **/
	private boolean storageAllowed = true;

	//added for use in store method..
	private ModelMBeanAttributeInfo currentads = null;

	private Date nomoreoftentimer = null;

	/** The first packet flag **/
	private boolean firstPacket = true;

	/** field separator used in the conf file **/
	private String fieldSeparator= " \t\n\r";

	/**
	 * The table that hols the persistent mbeans info as specified in
	 * the conf file
	 */
	Hashtable confFileTable = new Hashtable();

	/**
	 * The scheduler object used for timer persistent suppport for different
	 * attributes. Each OnTimer attribute gets scheduled by this scheduler.
	 */
	private static Scheduler scheduler = null;

	Hashtable adsValueTable = new Hashtable();

	PrintStream prints = null;

	//Implementing ModelMBeanNotificationBroadcaster
	NotificationBroadcasterSupport notifbroadcaster = null;
	NotificationBroadcasterSupport attrbroadcaster = null;
	Hashtable attrFilterMapTable = new Hashtable();

    String[] notiffields1 = new String[]{"name=General", "descriptorType=notification","severity=5",
									 "log=T", "displayName=jmx general"};

    String[] notiffields2 = new String[]{"name=AttributeChange", "descriptorType=notification", "severity=5",
                                         "log=T", "displayName=jmx attribute change"};

	private static int MAX_THREADS = 4;

	private static boolean modelMBeansStorage = true;

	private static boolean modelMBeansCaching = true;

	/**
	 * Constructs an RequiredModelMBean with an empty ModelMBeanInfo. After the
	 * RequiredModelMBean's MBeanInfo and Descriptors are customized, the
	 * RequiredModelMBean should be registered with the MBeanServer.
	 *
	 * @exception MBeanException The constructor of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException
	 */
	public RequiredModelMBean() throws MBeanException, RuntimeOperationsException
	{
		try
		{
			createLogger();
			log.trace("Before calling load method");
			load();
			log.trace("load called successfully");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Constructs a RequiredModelMBean object using ModelMBeanInfo passed in.
	 * The RequiredModelMBean must be instantiated, but not registered with the
	 * MBeanServer. After the RequiredModelMBean's MBeanInfo and Descriptors
	 * are customized, the RequiredModelMBean should be registered with the
	 * MBeanServer.
	 *
	 * @param mbi The ModelMBeanInfo object to be used by the RequiredModelMBean.
	 *
	 * @exception MBeanException The constructor of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException :
	 * 				The MBeanInfo passed in parameter is null or invalid.
	 */
	public RequiredModelMBean(ModelMBeanInfo mbi)
                   			throws MBeanException, RuntimeOperationsException
	{
		createLogger();

		if(mbi == null)
			throw new RuntimeOperationsException(new IllegalArgumentException("mbi"));

		this.mbeanInfo = mbi;

		try
		{
			log.trace("Before calling load method");
			load();
			log.trace("load called successfully");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Sets the instance handle of the object against which to execute all
	 * methods in this RequiredModelMBean management interface (ModelMBeanInfo
	 * and Descriptors). This setting can be overridden by setting the
	 * 'targetObject' field of the ModelMBeanOperationInfo's descriptor.
	 *
	 * @param mr Object that is the managed resource
	 *
	 * @param mr_type The type of reference for the managed resource. Can be:
	 * 				ObjectReference, Handle, IOR, EJBHandle, RMIReference.
	 * 				If the MBeanServer cannot process the mr_type passed in,
	 * 				an exception will be thrown. Current Implementation of
	 * 				RequiredModelMBean supports only ObjectReference.
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The managed resource or managed resoure type passed in
	 * 				parameter is null or invalid.
	 *
	 * @exception InstanceNotFoundException The managed resource object
	 * 				could not be found
	 *
	 * @exception InvalidTargetObjectType The managed resource type cannot
	 * 				be processed by the RequiredModelMBean or JMX Agent.
	 */
	public void setManagedResource(Object mr, String mr_type)
                        throws MBeanException,
                               RuntimeOperationsException,
                               InstanceNotFoundException,
                               InvalidTargetObjectTypeException
	{
		if(mr == null || mr_type ==  null)
			throw new RuntimeOperationsException(
							new IllegalArgumentException("Invalid mr or mr_type"));

		if(mr_type.equalsIgnoreCase(OBJECT_REFERENCE) == false)
			throw new InvalidTargetObjectTypeException(("Unsupported mr_tpe"));

		this.object = mr;
		this.objectType = mr_type;

		clazz = object.getClass();

		if(scheduler == null)
		{
			scheduler = Scheduler.createScheduler("JMX",MAX_THREADS);
			scheduler.start();
			log.trace("JMX Scheduler instantiated successfully");
		}

		//set the "class" field in ModelMBeanOperationInfo's & ModelMBeanConstructorInfo's
		//if it is not already set

		if(this.mbeanInfo != null)
		{
			ModelMBeanConstructorInfo[] constrs = (ModelMBeanConstructorInfo[])this.mbeanInfo.getConstructors();
			ModelMBeanOperationInfo[] opers = (ModelMBeanOperationInfo[])this.mbeanInfo.getOperations();

			if(constrs != null)
			{
				for(int i=0;i<constrs.length;i++)
					if(constrs[i].getDescriptor().getFieldValue("class") == null)
						constrs[i].getDescriptor().setField("class", clazz.getName());
			}

			if(opers != null)
			{
				for(int i=0;i<opers.length;i++)
					if(opers[i].getDescriptor().getFieldValue("class") == null)
						opers[i].getDescriptor().setField("class", clazz.getName());
			}
		}

		//to be deleted
		initializeAttributeDescriptorsField((ModelMBeanAttributeInfo[])mbeanInfo.getAttributes());
		log.trace("Attribute Descriptors Initialized");
	}

	/**
	 * Initializes a RequiredModelMBean object using ModelMBeanInfo passed in.
	 * The RequiredModelMBean should be instantiated, but not registered with
	 * the MBeanServer. After the RequiredModelMBean's MBeanInfo and
	 * Descriptors are customized, the RequiredModelMBean should be registered
	 * with the MBeanServer.
	 * <P>
	 * @param mbi The MBeanInfo object to be used by the RequiredModelMBean.
	 *
	 * @exception MBeanException The constructor of the MBeanInfo has return null
	 * 				or thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 *       		The MBeanInfo passed in parameter is null or invalid.
	 *
	 */
	public void setModelMBeanInfo(ModelMBeanInfo mbi)
                       		throws MBeanException, RuntimeOperationsException
	{
		if(mbi == null)
			throw new RuntimeOperationsException(new IllegalArgumentException("mbi"));

		this.mbeanInfo = mbi;
	}

	/**
	 * Locates the MBean in a persistent store and primes this instance of the
	 * MBean with the stored values.Any currently set values are overwritten.
	 *
	 * @throws MBeanException Wraps another exception or persistence is not supported
	 *
	 * @throws RuntimeOperationsException Wraps exceptions from the persistence mechanism
	 *
	 * @throws InstanceNotFoundException Could not find or load this MBean
	 * 				from persistent storage
	 */
	public void load()
		throws MBeanException, RuntimeOperationsException, InstanceNotFoundException
	{
		// Check for Persistence else return
		if(!findPersistent())
		{
			log.info("NO PERSISTENT DATA FOUND");
			return;
		}

		// Read the MbeanInfo and initialize attributes
		try
		{
			InputStream in=new FileInputStream(persistLocation+File.separator+persistName);
			ObjectInputStream data=new ObjectInputStream(in);

			mbeanInfo=(ModelMBeanInfo)data.readObject();
			initializeAttributeDescriptorsField((ModelMBeanAttributeInfo[])mbeanInfo.getAttributes());

			while(true)
			{
				try
				{
					String name = (String)data.readObject();
					Object value = data.readObject();

					if(value instanceof TabularData && value != null)
					{
						TabularData tdata = (TabularData)value;
						for(Enumeration e = tdata.enumerate();e.hasMoreElements();)
						{
							Object[] index = (Object[])e.nextElement();
							CompositeData comp = tdata.getRow(index);
							comp.setOperationType(CompositeData.CREATED);

							for(Enumeration ce = comp.enumerate();ce.hasMoreElements();)
							{
								String key = (String)ce.nextElement();
								comp.setModified(key, true);
							}
						}
					}

					log.debug("load(): Going to set " + value);
					setAttribute(new Attribute(name, value));
				}
				catch(Exception e1)
				{
					break;
				}
			}
		}
		catch(Exception e)
		{
		    throw new MBeanException(e);
		}
	}

	/**
	 * Writes the MBean in a persistent store.Should only be called by
	 * the <i> RequiredModelMBean</i> class to store itself according to
	 * persistence policy for the MBean.When used,it may be called with every
	 * setAttribute or on a periodic basis. Currently the store supports only
	 * the flat file persistence.
	 *
     * @throws MBeanException Wraps another exception or persistence is not supported
     *
     * @throws RuntimeOperationsException Wraps exceptions from the persistence mechanism
     *
     * @throws InstanceNotFoundException Could not find or load this MBean
	 * 				from persistent storage
	 */
	public void store()
			throws MBeanException,
				   RuntimeOperationsException,
				   InstanceNotFoundException
	{
	    try
		{
			Descriptor mmbDesc=mbeanInfo.getMBeanDescriptor();

			persistLocation = (String)mmbDesc.getFieldValue("persistLocation");
			persistName     = (String)mmbDesc.getFieldValue("persistName");

			log.trace("Location="+persistLocation+" persistName="+persistName);

			if(!modelMBeansStorage)
				storageAllowed = modelMBeansStorage;

			if(!storageAllowed)
				return;

			ModelMBeanAttributeInfo[] attrs = (ModelMBeanAttributeInfo[])mbeanInfo.getAttributes();
			Descriptor attributeDescriptor  = currentads.getDescriptor();
			String  persistPolicy=(String)attributeDescriptor.getFieldValue(DescriptorSupport.PERSIST_POLICY);

			//log.debug("Grrrrrr"+attributeDescriptor.getFieldValue(DescriptorSupport.LAST_UPDATED_TIMESTAMP));

			//if the attribute is not writable or persist policy is null
			if(!(currentads.isWritable())||persistPolicy==null)
			{
				log.trace("attribute is not writable or persist policy is null");
				return;
			}

			//if persist policy is never
			if(persistPolicy.equalsIgnoreCase("Never"))
			{
				log.trace(" persist policy is never");
				return;
			}
			//if persist policy is OnTimer
			else if(persistPolicy.equalsIgnoreCase("OnTimer"))
			{
				log.trace(" persist policy is OnTimer");

				//get The Persist Period
				String period      = (String)attributeDescriptor.getFieldValue(DescriptorSupport.PERSIST_PERIOD);
				long persistPeriod = Long.parseLong(period);

				//get The CurrentTime
				long currTime=new Date().getTime();

				Date when=new Date(currTime+persistPeriod);
				//schedule Task
				scheduler.scheduleTask(new UpdateTimer(this,attributeDescriptor,when.getTime(),persistPeriod),when);
			}
			//if persist policy is OnUpdate
			else if(persistPolicy.equalsIgnoreCase("OnUpdate"))
			{
				log.trace(" persist policy is OnUpdate");
				OnUpdate(attributeDescriptor);
			}
			//if persist policy is NoMoreOftenThan
			else if(persistPolicy.equalsIgnoreCase("NoMoreOftenThan"))
			{
				log.trace(" persist policy is NoMoreOftenThan");
				//get the time period
				String period      = (String)attributeDescriptor.getFieldValue(DescriptorSupport.PERSIST_PERIOD);
				long persistPeriod = Long.parseLong(period);
				log.trace("Persist Period="+persistPeriod);

				//get the last updated time stamp
				String lastTime=(String)attributeDescriptor.getFieldValue(DescriptorSupport.LAST_UPDATED_TIMESTAMP);
				long lastUpdatedTime=new Date(Long.parseLong(lastTime)).getTime();
				log.trace("Last Updated Time="+lastUpdatedTime);

				//get the current  Time Period
				long currTime=new Date().getTime();
				log.trace("Current Time="+currTime);

				if(currTime >= (lastUpdatedTime+persistPeriod))
				{
				    log.trace("currTime is greater than (lastUpdatedTime+persistPeriod)");
				    OnUpdate(attributeDescriptor);
				}
			}
	    }
		catch(Exception e)
		{
			log.error(e.getMessage());
			throw new MBeanException(e);
	    }
	} //store

	/**
	 * This method discovers the attributes and operations this MBean
	 * exposes for management.
	 *
     * @return An instance of MBeanInfo allowing to retrieve all attributes
	 * 				and operations of this MBean.
	 */
	public MBeanInfo getMBeanInfo()
	{
		if(mbeanInfo==null)
		{
			log.debug("MBeanInfo is null");
			mbeanInfo= createDefaultModelMBeanInfo();
			log.debug("Default Model Mbean Created");
		}

		return (MBeanInfo)mbeanInfo;
	}

	/**
	 * Gets the value of a specific attribute of this MBean.
	 *
	 * @param attribute A String specifying the name of the attribute to be retrieved.
	 *
     * @return The value of the retrieved attribute.
	 */
	public Object getAttribute(String attribute)
							throws AttributeNotFoundException,
									MBeanException,
									ReflectionException
	{
		return getAttribute(attribute, true);
	}

	/**
	* Enables the values of several attributes of this MBean.
	*
	* @param attributes A list of attributes to be retrieved.
	*
    * @return The value of the retrieved attributes as attributeList.
	*/
	public AttributeList getAttributes(String[] attributes)
	{
		AttributeList toRet = null;
		if(attributes == null)
			throw new RuntimeOperationsException(new RuntimeException("invalid method param"));

		toRet = new AttributeList();
		for(int i = 0; i<attributes.length;i++)
		{
			Attribute attr = null;

			try
			{
				attr = new Attribute(attributes[i], getAttribute(attributes[i]));
			}
			catch(Exception e){ }
			toRet.add(attr);
		}

		return toRet;
	}

	/**
	 * Invokes an action on this MBean.
	 *
	 * @param actionName The name of the action to be invoked.
	 *
     * @param params An array containing the parameters to be set when
	 * 				the action is invoked
	 *
     * @param signature An array containing the signature of the action.
	 * 				The class objects will be loaded using the same
	 * 				class loader as the one used for loading the MBean on
	 * 				which the action was invoked.
	 *
     * @return The object returned by the action, which represents the result
	 * 				of invoking the action on the specified MBean.
	 *
	 * @throws MBeanException Wraps an exception thrown by the MBean's invoked method.
	 *
	 * @throws ReflectionException Wraps an java.lang.Exception thrown while
	 * 				trying to invoke the method.
	 */
	public Object invoke(String actionName, Object[] params, String[] signature)
									throws MBeanException, ReflectionException
	{
		if(signature == null)
      		    signature = new String[0];

		boolean methodFound = true;

		try
		{
            Class[] sigClass = null;
            //if(signature != null)
            //{
            sigClass = new Class[signature.length];
            for(int i=0;i<signature.length;i++)
			{
                //sigClass[i] = Class.forName(signature[i]);
                try
				{
                    sigClass[i]=Thread.currentThread().getContextClassLoader().loadClass(signature[i]);
                }
				catch(Exception ee)
				{
                    sigClass[i] = getProperClass(signature[i]);
                    if(sigClass[i] == null && params[i] != null)
					{
                        sigClass[i] = params[i].getClass().getClassLoader().loadClass(signature[i]);
                    }
                }
                //System.out.println(" The class object "+sigClass[i]);
            }
            //}
            if(this.getClass().getMethod(actionName, sigClass) == null)
			{
                methodFound = false;
            }
		}
		catch(Exception e)
		{
			methodFound = false;
		}

		/*if(methodFound && (actionName.equals("setManagedResource") ||
		   actionName.equals("setModelMBeanInfo") ||
		   actionName.equals("load") ||
		   actionName.equals("store") ||
		   actionName.equals("addAttributeChangeNotificationListener") ||
		   actionName.equals("removeAttributeChangeNotificationListener") ||
		   actionName.equals("sendAttributeChangeNotification") ||
		   actionName.equals("sendNotification") ||
		   actionName.equals("addNotificationListener") ||
		   actionName.equals("getNotificationInfo") ||
		   actionName.equals("removeNotificationListener")))*/
		if(methodFound)
		{
			return invoke(actionName, params, signature, true);
		}

		//if(signature != null && signature.length == 1)
		if(signature.length==1)
		{
			ModelMBeanAttributeInfo[] adss = (ModelMBeanAttributeInfo[])mbeanInfo.getAttributes();
			ModelMBeanAttributeInfo attr = null;
			for(int i=0;i<adss.length;i++)
			{
				attr = adss[i];

				if(("set"+attr.getName()).equals(actionName) &&
				   attr.getType().equals(signature[0]))
				{
					try
					{
						setAttribute(new Attribute(attr.getName(), params[0]));
					}
					catch(javax.management.AttributeNotFoundException ae)
					{
						throw new MBeanException(ae);
					}
					catch(javax.management.InvalidAttributeValueException ie)
					{
						 throw new MBeanException(ie);
					}
					//throw new MBeanException(new ServiceNotFoundException("Exception during invokation of operation : "+actionName));
					return null;
				}
			}
		}

		//here check for operation match with managed resource
		log.debug("action name="+actionName);
                /*
		ModelMBeanOperationInfo operInfo = this.mbeanInfo.getOperation(actionName);
		if(operInfo == null)
			throw new MBeanException(new ServiceNotFoundException("Exception during invokation of operation : "+actionName));

		MBeanParameterInfo[] paramsInfo = operInfo.getSignature();
		//if(signature == null || (paramsInfo.length != signature.length))
		if(paramsInfo.length!=signature.length)
			throw new MBeanException(new ServiceNotFoundException("Exception during invocation of operation : "+actionName));

		for(int i=0;i<paramsInfo.length;i++)
		{
			if(!paramsInfo[i].getType().equals(signature[i]))
				throw new MBeanException(new ServiceNotFoundException("Exception during invokation of operation : "+actionName));
		}
		*/

        // To Support Method Overloading - Ramesh
        boolean operfound = false;
        ModelMBeanOperationInfo operInfo = null;
        MBeanOperationInfo[] operInfos = this.mbeanInfo.getOperations();
        for(int k=0; k<operInfos.length; k++)
		{
            operInfo = (ModelMBeanOperationInfo)operInfos[k];
            if(!operInfo.getName().equals(actionName))
			{
                continue;
            }
            MBeanParameterInfo[] paramsInfo = operInfo.getSignature();
            if(paramsInfo.length != signature.length)
			{
                continue;
            }
            int tmpcount = 0;
            for(int i=0;i<paramsInfo.length;i++)
			{
                if(!paramsInfo[i].getType().equals(signature[i]))
				{
                    break;
                }
				else
				{
                    tmpcount++;
                }
            }
            if(tmpcount == paramsInfo.length)
			{
                operfound = true;
                break;
            }
        }

        if(!operfound)
		{
            throw new MBeanException(new ServiceNotFoundException("Exception during invokation of operation : "+actionName));
        }
        // Fix for Method overloading support ends

		Descriptor desc = operInfo.getDescriptor();

		if(desc.getFieldValue(DescriptorSupport.TARGET_OBJECT) != null &&
		   desc.getFieldValue(DescriptorSupport.TARGET_TYPE) != null &&
		   ((String)desc.getFieldValue(DescriptorSupport.TARGET_TYPE)).equalsIgnoreCase("objectReference"))
		{

			return invokeFriend(actionName, params, signature,
								desc.getFieldValue(DescriptorSupport.TARGET_OBJECT));
		}


		return invoke(actionName, params, signature, false);
	}

	/**
	 * Sets the value of a specific attribute of this MBean.
	 *
     * @param attribute The identification of the attribute to be set and
	 * 				the value it is to be set to.
	 *
     * @throws AttributeNotFoundException - The specified attribute is
	 * 				not accessible in the MBean.
	 *
	 * @throws InvalidAttributeValueException - The specified value for
	 * 				the attribute is not valid.
 	 *
 	 * @throws MBeanException - Wraps an exception thrown by the MBean's setter.
	 *
	 * @throws ReflectionException - Wraps an java.lang.Exception thrown
	 * 				while trying to invoke the setter.
	 *
	 * @throws RuntimeOperationsException - Wraps an IllegalArgumentException:
	 * 				The object name in parameter is null or the attribute
	 * 				in parameter is null.
	 */
	public void setAttribute(Attribute attribute)
						throws AttributeNotFoundException,
								InvalidAttributeValueException,
								MBeanException,
								ReflectionException
	{
        log.trace("setAttribute() entering");

		ModelMBeanAttributeInfo[] adss = (ModelMBeanAttributeInfo[])mbeanInfo.getAttributes();
		ModelMBeanOperationInfo[] odss = (ModelMBeanOperationInfo[])mbeanInfo.getOperations();

		ModelMBeanAttributeInfo attr = null;
		ModelMBeanOperationInfo oper = null;

		Descriptor ads = null;
		Descriptor ods = null;

		for(int i=0;i<adss.length;i++)
		{
			attr = adss[i];
			ads = attr.getDescriptor();
			if(ads.getFieldValue(DescriptorSupport.NAME).equals(attribute.getName()))
			{

				if(ads.getFieldValue(DescriptorSupport.SET_METHOD) != null);
				{
					for(int j=0;j<odss.length;j++)
					{
						oper = odss[j];
						ods = oper.getDescriptor();
						if(((String)ods.getFieldValue(DescriptorSupport.NAME)).equals(ads.getFieldValue(DescriptorSupport.SET_METHOD)) )
							break;
						else
						{
							oper = null;
							ods = null;
						}
					}
				}

				break;
			}
			else
			{
				attr = null;
				ads = null;
			}
		}

		Object value = null;
		if(ods != null)
		{
			value = attribute.getValue();

			Attribute oldattr = new Attribute(attribute.getName(), adsValueTable.get(ads));

			Object tempValue = adsValueTable.get(ads);
			log.trace("before value");
			log.trace("DD"+DescriptorSupport.VALUE);
			log.trace("ads="+ads);

			Object o  = ads.getFieldValue(DescriptorSupport.VALUE);//.toString();
			String tempValueString=null;
			if(o!=null)tempValueString=o.toString();
			//String tempValueString  = ads.getFieldValue(DescriptorSupport.VALUE).toString();
			log.trace("after value");

			adsValueTable.put(ads, value);
			ads.setField(DescriptorSupport.VALUE, value.toString());

			Hashtable tempTable = new Hashtable();
			boolean isTabularCreate = false;
			//Added for sending only changed rows in TabularData and changed columns in the
			//CompositeData
			if(value instanceof TabularData)
			{
				TabularData data = (TabularData)value;
				TabularParameterInfo info = data.getTabularParameterInfo();
				String[] indexNames = info.getIndexNames();

				for(Enumeration e = data.enumerate();e.hasMoreElements();)
				{
					Object[] index = (Object[])e.nextElement();
					CompositeData comp = data.getRow(index);
					if(comp.getOperationType().equals(CompositeData.NOCHANGES))
					{
						tempTable.put(index, data.deleteRow(index));
					}
					else if(comp.getOperationType().equals(CompositeData.MODIFIED) ||
							comp.getOperationType().equals(CompositeData.CREATED))
					{
						if(comp.getOperationType().equals(CompositeData.CREATED))
							isTabularCreate = true;
						for(Enumeration ce = comp.enumerate();ce.hasMoreElements();)
						{
							String key = (String)ce.nextElement();
							if(!comp.isModified(key))
							{
								if(isInArray(indexNames, key))
									continue;

								Hashtable ttt = (Hashtable)tempTable.get(index);
								if(ttt == null)
									ttt = new Hashtable();
								ttt.put(key, comp.deleteDataItem(key));
								tempTable.put(index, ttt);
							}
						}
					}
				}
			}//end

			try
			{
				invoke((String)ods.getFieldValue(DescriptorSupport.NAME), new Object[]{value}, new String[]{
					   oper.getSignature()[0].getType()}, false);

				Attribute newattr = new Attribute(attribute.getName(), value);
				sendAttributeChangeNotification(oldattr, newattr);

			}
			catch(IllegalArgumentException iaex)
			{
				throw new InvalidAttributeValueException(iaex.getMessage());
			}
			catch(javax.management.MBeanException me)
			{
				//me.printStackTrace();
				restoreTabularData(value, tempTable);

				if(tempValue != null)
					adsValueTable.put(ads, tempValue);
				if(tempValueString != null)
					ads.setField(DescriptorSupport.VALUE, tempValueString);

				throw me;
			}
			catch(javax.management.ReflectionException re)
			{
				//re.printStackTrace();
				restoreTabularData(value, tempTable);

				if(re.getTargetException() instanceof java.lang.NoSuchMethodException)
					throw new AttributeNotFoundException(attribute.getName());

				if(tempValue != null)
					adsValueTable.put(ads, tempValue);
				if(tempValueString != null)
					ads.setField(DescriptorSupport.VALUE, tempValueString);

				throw re;
			}

			restoreTabularData(value, tempTable);

			String lutime = null;

			if(enableStorage)
			{

				currentads = attr;
				try
				{
					store();
					lutime = (String)ads.getFieldValue(DescriptorSupport.LAST_UPDATED_TIMESTAMP);
				}
				catch(Exception ex)
				{
					throw new MBeanException(ex);
				}
			}

			//if(isTabularCreate)
			if(value instanceof TabularData)
			{
				String ctl = (String)ads.getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT) + "000";
				try
				{
					long longCtl = Long.parseLong(ctl);
					ads.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ (new Date().getTime() - longCtl - 1));
				}
				catch(Exception e){e.printStackTrace();}
				getAttribute(attribute.getName(), false);
				if(lutime != null)
					ads.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ lutime);
			}

			return ;
		}
		else if(ads == null)
		{
			throw new AttributeNotFoundException("Attribute \"" + attribute.getName() + "\" not found");
		}
		else if((value = ads.getFieldValue(DescriptorSupport.VALUE)) != null)
		{
			//value = adsValueTable.get(ads);

			adsValueTable.put(ads, value);
			ads.setField(DescriptorSupport.VALUE, value);
			return;
		}

		String defaultValue = (String)ads.getFieldValue(DescriptorSupport.DEFAULT);
		if(defaultValue == null)
			throw new AttributeNotFoundException("Attribute has no default value"+ ads.getFieldValue(DescriptorSupport.NAME));

		ads.setField(DescriptorSupport.DEFAULT, attribute.getValue().toString());

		/*
		try
		{
			Class e = object.getClass();

			Class parameterTypes[]=new Class[1];
			parameterTypes[0] = attribute.getValue().getClass();

		    Method attrMethod = e.getMethod("set"+attribute.getName(),parameterTypes);

			attrMethod.invoke(object, new Object[]{attribute.getValue()} );
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		*/
	}

	/**
	* Sets the values of several attributes of this MBean.
	*
	* @param attributes A list of attributes: The identification of the
	* 				attributes to be set and the values they are to be set to.
	*
    * @return The list of attributes that were set, with their new values.
	*/
	public AttributeList setAttributes(AttributeList attributes)
	{
		log.trace("setAttributes");
		if(attributes == null)
			return null;

		Object[] array = attributes.toArray();

		if(array == null)
			return attributes;

		for(int i = 0; i<array.length;i++)
		{
			Attribute attr = null;
			try
			{
				attr = (Attribute)array[i];
			}
			catch(ClassCastException ce)
			{
				continue;
			}

			try
			{
				setAttribute(attr);
			}
			catch(Exception e){  }
		}

		return attributes;
	}

	/**
	 * Enables a couple (listener,handback) for a registered MBean to be added.
	 *
	 * Specified by :
	 * 				addNotificationListener in interface NotificationBroadcaster
	 *
 	 * @param listener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @param filter The filter object. If not specified, no filtering will be
	 * 				performed before handling notifications.
	 *
     * @param handback The context to be sent to the listener when a
	 * 				notification is emitted.
	 *
	 * @throws java.lang.IllegalArgumentException Listener parameter is null.
	 */
	public void addNotificationListener(NotificationListener listener,
										NotificationFilter filter,
										Object handback)
										throws IllegalArgumentException

	{
		log.trace("add Notification Listener");

		if( listener == null)
			throw new IllegalArgumentException("Notif Listener is null");

		if(notifbroadcaster == null)
		{
			notifbroadcaster =  new NotificationBroadcasterSupport();
		}

		notifbroadcaster.addNotificationListener(listener, filter, handback);
	}


	/**
	 * Enables a listener for an MBean to be removed. All couple
	 * (listener, handback) are removed.
	 *
	 * Specified by:
	 * 				removeNotificationListener in interface NotificationBroadcaster
	 *
	 * @param listener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @throws ListenerNotFoundException The listener is not registered in the MBean.
	 */
	public void removeNotificationListener(NotificationListener listener)
                                throws ListenerNotFoundException
	{
		log.trace("remove Notification Listener");

		if( listener == null || notifbroadcaster == null)
			throw new ListenerNotFoundException("Notif Listener is null");

		notifbroadcaster.removeNotificationListener(listener);
	}

	/**
	 * Returns a NotificationInfo object containing the name of the Java class
	 * of the notification and the notification types sent.
	 * This getNotificationInfo method will inturn call the original
	 * instrumentation object's getNotificationInfo method if the object is
	 * an instanceof NotificationBroadcaster.
	 */
	public MBeanNotificationInfo[] getNotificationInfo()
	{
		MBeanNotificationInfo[] notifications = mbeanInfo.getNotifications();
		ModelMBeanNotificationInfo[] toRet = null;

		if(notifications == null)
			toRet = new ModelMBeanNotificationInfo[2];
		else
			toRet = new ModelMBeanNotificationInfo[notifications.length + 2];

		Descriptor d1 = new DescriptorSupport(notiffields2);
		Descriptor d2 = new DescriptorSupport(notiffields2);

		Descriptor descr = null;
		try
		{
        	descr = mbeanInfo.getMBeanDescriptor();
		}
		catch(Exception e){}

		if(d1.getFieldValue(DescriptorSupport.LOG_FILE) == null &&
                   descr.getFieldValue(DescriptorSupport.LOG_FILE) != null)
                    d1.setField(DescriptorSupport.LOG_FILE, descr.getFieldValue(DescriptorSupport.LOG_FILE));

		if(d2.getFieldValue(DescriptorSupport.LOG_FILE) == null &&
                   descr.getFieldValue(DescriptorSupport.LOG_FILE) != null)
                    d2.setField(DescriptorSupport.LOG_FILE, descr.getFieldValue(DescriptorSupport.LOG_FILE));

		if(descr.getFieldValue(DescriptorSupport.LOG) != null)
                    d1.setField(DescriptorSupport.LOG, descr.getFieldValue(DescriptorSupport.LOG));

		if(descr.getFieldValue(DescriptorSupport.LOG) != null)
                    d2.setField(DescriptorSupport.LOG, descr.getFieldValue(DescriptorSupport.LOG));

		toRet[0] = new ModelMBeanNotificationInfo(new String[] {"jmx.modelmbean.general"},
                                                          "General",
                                                          "general notification",
                                                          d1);

		toRet[1] = new ModelMBeanNotificationInfo(new String[] {"jmx.attribute.change"},
                                                          "AttributeChange",
                                                          "attribute change notification",
                                                          d2);

		if(notifications != null)
		{
            for (int i=0,j=2; i < notifications.length; i++,j++)
			{
				toRet[j] = (ModelMBeanNotificationInfo)(notifications[i]);
			}
		}

		return toRet;

		/*
		if( !(object instanceof NotificationBroadcaster))
			return null;

		return ((NotificationBroadcaster)object).getNotificationInfo();
		*/
	}

	/**
	 * Register3s an object which implements the NotificationListener interface
	 * as a listener for AttributeChangeNotifications.  This object's
	 * 'handleNotification()' method will be invoked when any
	 * attributeChangeNotification is issued through or by the MBean.  This
	 * does not include other Notifications.  They must be registered for
	 * independently. An AttributeChangeNotification will be generated for this
	 * attributeName.
	 *
	 * @param inlistener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @param inAttributeName The name of the MBean attribute for which to
	 * 				receive change notifications. If null, then all attribute
	 * 				changes will cause an attributeChangeNotification to be issued.
	 *
	 * @param inhandback The context to be sent to the listener with the
	 * 				notification when a notification is emitted.
	 *
	 * @throws MBeanException  Wraps an exception thrown by this method
	 *
	 * @throws RuntimeOperationsException To wrap the Run time Exceptions
	 *
	 * @throws IllegalArgumentException Listener is null or attributeName is null.
	 */
	public void addAttributeChangeNotificationListener(NotificationListener inlistener,
													   String inAttributeName,
													   Object inhandback)
										throws MBeanException,
											   RuntimeOperationsException,
											   IllegalArgumentException
	{
		log.trace("addAttributeChangeNotificationListener");

		if( inlistener == null)
			throw new IllegalArgumentException("Attr Notif Listener is null");

		if(attrbroadcaster == null)
			attrbroadcaster =  new NotificationBroadcasterSupport();

		AttributeChangeNotificationFilter infilter = (AttributeChangeNotificationFilter)attrFilterMapTable.get(inlistener);
		if( infilter == null)
		{
			infilter = new AttributeChangeNotificationFilter();
			attrFilterMapTable.put(inlistener, infilter);
		}
		infilter.enableAttribute(inAttributeName);

		attrbroadcaster.addNotificationListener(inlistener, infilter, inhandback);
	}

	/**
	 * Removes a listener for attributeChangeNotifications from the MBean.
	 *
	 * @param inlistener The listener name which was handling notifications
	 * 				emitted by the registered MBean. This method will remove
	 * 				all information related to this listener.
	 *
	 * @param inAttributeName The attribute for which the listener no longer
	 * 				wants to receive attributeChangeNotifications.
	 *
	 * @throws MBeanException  Wraps an exception thrown while
	 * 				removing AttributeChangeNotificationListener
	 *
	 * @throws RuntimeOperationsException Wraps the exceptions thrown in the
	 * 				run time
	 *
	 * @exception ListenerNotFoundException The couple (listener,handback) is
	 * 				not registered in the MBean. The exception message contains
	 * 				either "listener", "handback" or the object name
	 * 				depending on which object cannot be found.
	 */
	public void removeAttributeChangeNotificationListener(NotificationListener inlistener,
														  String inAttributeName)
										throws MBeanException,
											   RuntimeOperationsException,
											   ListenerNotFoundException
	{
		log.trace("removeAttributeChangeNotificationListener");

		if( inlistener == null || notifbroadcaster == null)
			throw new ListenerNotFoundException("Notif Listener is null");

		AttributeChangeNotificationFilter infilter = (AttributeChangeNotificationFilter)attrFilterMapTable.get(inlistener);
		infilter.disableAttribute(inAttributeName);

		//attrbroadcaster.removeNotificationListener(inlistener);
	}

	/**
	 * Sends a Notification which is passed in to the registered Notification
	 * listeners on the ModelMBean as a jmx.modelmbean.general notification.
	 *
	 * @param ntfyObj The notification which is to be passed to the
	 * 				'handleNotification' method of the listener object.
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The Notification object passed in parameter is null or invalid.
	 */
	public void sendNotification(Notification ntfyObj)
							throws MBeanException, RuntimeOperationsException
	{
		log.trace("Sending Notification [" + ntfyObj + "]");

		if(ntfyObj == null)
        	throw new RuntimeOperationsException(new IllegalArgumentException("notif object is null"));

		if(notifbroadcaster == null)
		{
			log.trace("Notification broadcaster is NULL and hence not sending notification [" + ntfyObj + "]");
			return;
		}

		MBeanNotificationInfo notifinfo = null;
		if((notifinfo = getNotifInfo(ntfyObj)) != null)
		{
			//throw new RuntimeOperationsException(new IllegalArgumentException(
			//             "Notification undefined in the model mbean")); */
			logTheNotification(ntfyObj, notifinfo);
		}

		log.trace("$$$ Before sending [" + ntfyObj + "] using notification broadcaster");
		notifbroadcaster.sendNotification(ntfyObj);
		log.trace("$$$$ AFTER sent [" + ntfyObj + "] using notification broadcaster");
	}

	/**
	 * Sends a Notification which contains the text string that is passed in
	 * to the registered Notification listeners on the ModelMBean.
	 *
	 * @param ntfyText The text which is to be passed in the Notification to the
	 * 				'handleNotification' method of the listener object.
	 * 				the constructed Notification will be:
	 * 				type        "jmx.modelmbean.general"
	 * 				source      this ModelMBean instance
	 * 				sequence    1
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The Notification text string passed in parameter is null or invalid.
	 */
	public void sendNotification(String ntfyText) throws MBeanException,
														 RuntimeOperationsException
	{
		//to do :: log the notification sent.
		if (ntfyText == null)
		{
			throw new RuntimeOperationsException(
					new IllegalArgumentException("Notification message should not be empty or null"),
					"Exception occured trying to send a text notification from a ModelMBean");
		}

		Notification ntfyObj = new Notification("jmx.modelmbean.general",
                                                        this,
                                                        1,
                                                        new Date().getTime(),
                                                        ntfyText);
		log.trace("$$$$ Notification is " + ntfyObj);
		sendNotification(ntfyObj);
		log.trace("$$$ Sent Notification is " + ntfyObj);
	}

	/**
	 * Sends an attributeChangeNotification which is passed in to the registered
	 * attributeChangeNotification listeners on the ModelMBean.
	 *
	 * @param ntfyObj The notification which is to be passed to the
	 * 				'handleNotification' method of the listener object.
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The Notification object passed in parameter is null or invalid.
	 */
	public void sendAttributeChangeNotification(AttributeChangeNotification ntfyObj)
            throws MBeanException,
                   RuntimeOperationsException
	{
		//to do :: log the notification sent.

		if(ntfyObj == null)
			throw new RuntimeOperationsException(new IllegalArgumentException("notif object is null"));

		if(attrbroadcaster == null)
			return;

		MBeanNotificationInfo notifinfo = null;
		if((notifinfo = getNotifInfo(ntfyObj)) == null)
			throw new RuntimeOperationsException(new IllegalArgumentException(
				"Notification undefined in the model mbean"));
		logTheNotification(ntfyObj, notifinfo);

		attrbroadcaster.sendNotification(ntfyObj);
	}

	/*
     * To get the number of threads that this scheduler controls. Default value is 4
	 * @return Maximum number of threads controlled by this scheduler
     */
	/*
	public static int getMaxThreads()
	{
		return MAX_THREADS ;
    }
	*/

	/*
     * To set the number of threads that this scheduler controls
	 * @param maxThreads The number of threads that this scheduler controls
     */
    /*
	public static void setMaxThreads(int maxThreads)
	{
		MAX_THREADS = maxThreads;
    }
    */

	/**
	 * Sends an attributeChangeNotification which contains the old value and
	 * new value for the attribute to the registered
	 * AttributeChangeNotification listeners on the ModelMBean.
	 *
	 * @param inOldVal The origional value for the Attribute
	 *
	 * @param inNewVal The current value for the Attribute
	 * <p>
	 * <p>
	 * <PRE>
	 * The constructed attributeChangeNotification will be:
	 * type        "jmx.attribute.change"
	 * source      this ModelMBean instance
	 * sequence    1
	 * attributeName oldValue.getName()
	 * attributeType oldValue's class
	 * attributeOldValue oldValue.getValue()
	 * attributeNewValue newValue.getValue()
	 * </PRE>
	 * <p>
	 *
	 * @exception MBeanException to wrapper implementation exceptions
	 *
	 * @exception RuntimeOperationsException to wrapper IllegalArgumentExceptions.
	 */
	public void sendAttributeChangeNotification(Attribute inOldVal, Attribute inNewVal)
											throws MBeanException,
												   RuntimeOperationsException
	{
		//to do :: log the notification sent.

		if(inOldVal == null || inNewVal ==null)
			throw new RuntimeOperationsException(new IllegalArgumentException("param is null"));

		String attrName = inOldVal.getName();
		String attrType = null;

		if(inOldVal.getValue() != null)
			attrType = inOldVal.getValue().getClass().getName();

		AttributeChangeNotification ntfyObj = new AttributeChangeNotification(
													  this,
													  1,
													  new Date().getTime(),
													  "Attribute changed...",
													  attrName,
													  attrType,
													  inOldVal.getValue(),
													  inNewVal.getValue());

		sendAttributeChangeNotification(ntfyObj);
	}

	//--------------------------- Inner class -----------------------------//

    // Call Back
    class UpdateTimer implements Runnable
	{
		RequiredModelMBean rmm=null;
		Descriptor attributeDescriptor=null;
		long when=0l;
		long persistPeriod=0l;

		UpdateTimer(RequiredModelMBean rmm,
					Descriptor attributeDescriptor,
					long when,
					long persistPeriod)
		{
		    this.rmm=rmm;
		    this.attributeDescriptor=attributeDescriptor;
		    this.when=when;
		    this.persistPeriod=persistPeriod;
		}

		public void run()
		{
		    log.trace(Thread.currentThread().getName());

		    try
			{
				rmm.OnUpdate(attributeDescriptor);

				scheduler.scheduleTask(new UpdateTimer(rmm,attributeDescriptor,new Date(when+persistPeriod).getTime(),persistPeriod),new Date(when+persistPeriod));
				log.info("SCHEDULER scheduled Task at"+new Date(when+persistPeriod));
		    }
			catch(Exception e)
			{
				log.error(e.getMessage());
		    }
		}
    }

	//--------------------------- Package methods ---------------------------//

	/**
	 * Gets the value of a specific attribute of this MBean.
	 *
	 * @param attribute A String specifying the name of the attribute to be retrieved.
	 *
	 * @return The value of the retrieved attribute.
	 */
	Object getAttribute(String attribute, boolean isFromServer)
							throws AttributeNotFoundException,
									MBeanException,
									ReflectionException
	{
		log.trace("getAttribute called in RMB ******");

		ModelMBeanAttributeInfo[] adss = (ModelMBeanAttributeInfo[])mbeanInfo.getAttributes();
		ModelMBeanOperationInfo[] odss = (ModelMBeanOperationInfo[])mbeanInfo.getOperations();

		ModelMBeanAttributeInfo attr = null;
		ModelMBeanOperationInfo oper = null;

		Descriptor ads = null;
		Descriptor ods = null;

		for(int i=0;i<adss.length;i++)
		{
			attr = adss[i];
			ads = attr.getDescriptor();
			if(((String)ads.getFieldValue(DescriptorSupport.NAME)).equalsIgnoreCase(attribute))
			{
				if(ads.getFieldValue(DescriptorSupport.GET_METHOD) != null);
				{
					for(int j=0;j<odss.length;j++)
					{
						oper = odss[j];
						ods = oper.getDescriptor();
						if(((String)ods.getFieldValue(DescriptorSupport.NAME)).equalsIgnoreCase(
									(String)ads.getFieldValue(DescriptorSupport.GET_METHOD)) )
							break;
						else
						{
							oper = null;
							ods = null;
						}
					}
				}

				break;
			}
			else
			{
				attr = null;
				ads = null;
			}
		}

		if(ads == null)
			throw new AttributeNotFoundException(attribute);

		Object value = null;

		try
		{
			if(ods != null)
			{
				String ctl = (String)ads.getFieldValue(DescriptorSupport.CURRENCY_TIME_LIMIT) + "000";
				String lut = (String)ads.getFieldValue(DescriptorSupport.LAST_UPDATED_TIMESTAMP);

				long ctlValue = -1;
				if(ctl != null)
					ctlValue = Long.parseLong(ctl);

				long lutValue = -1;
				try{
				if(lut != null)
					lutValue = Long.parseLong(lut);
				}catch(NumberFormatException nfe){
					//added by hyther
					//just throw the nfe
					//throw nfe;
					//nfe.printStackTrace();
					log.error("NumberFormatException:",nfe);
				}


				if((ctlValue > 0 || ctlValue == -1) && lutValue != -1)
				{
					long currTime = new Date().getTime();
					log.debug("currTime = "+currTime);

					if((currTime < (lutValue + ctlValue)) || ctlValue == -1)
					{
						value = adsValueTable.get(ads);
						String strVal = null;
						if(value != null)
							strVal = value.toString();

						try
						{
							if(strVal != null)
								value = returnDefaultValue(value, attr.getType());
						}
						catch(Exception ex)
						{
							throw new MBeanException(ex);
						}

						if(value == null ||
							(attr.getType().startsWith("[L") &&
							 attr.getType().endsWith(";")))
						{
							value = invoke((String)ods.getFieldValue(DescriptorSupport.NAME), new Object[0], null, false);
							if(value == null)
								throw new MBeanException(new Exception("value is null"));

							ads.setField(DescriptorSupport.VALUE, value.toString());
							adsValueTable.put(ads, value);
							log.debug("# Value = "+value.toString());
						}
						//ads.setField("lastUpdatedTimeStamp", ""+ (new Date().getTime()));
						return value;
					}
				}


				value = invoke((String)ods.getFieldValue(DescriptorSupport.NAME), new Object[0], null, false);
				if(value == null)
				{
					return null;
				}
				adsValueTable.put(ads, value);
				ads.setField(DescriptorSupport.VALUE, value.toString());
				if(isFromServer)
					ads.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ (new Date().getTime()));
				return value;
			}
			else if((value = ads.getFieldValue(DescriptorSupport.VALUE)) != null)
			{
				adsValueTable.put(ads, value);
				//value = adsValueTable.get(ads);
				String strVal = value.toString();
				//String strVal = ads.getField("value");

				try
				{
					//ads.setField("lastUpdatedTimeStamp", ""+ (new Date().getTime()));
					return returnDefaultValue(value, attr.getType());
				}
				catch(Exception ex)
				{
					throw new MBeanException(ex);
				}
			}

		}
		catch(NullPointerException npe)
		{
			log.debug("Attribute = "+attribute);
			log.debug("Object = "+object.getClass().getName());
			throw npe;
		}

		String defaultValue = (String)ads.getFieldValue(DescriptorSupport.DEFAULT);

		if(defaultValue == null)
			throw new AttributeNotFoundException("Attribute has no default value");

		try
		{
			//ads.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ (new Date().getTime()));
			return returnDefaultValue(defaultValue, attr.getType());
		}
		catch(Exception e)
		{
			throw new MBeanException(e);
		}
	}

	//--------------------------- Private methods ---------------------------//

	/**
	 * This allows all the Model MBeans storage control.
	 *
	 * @param  modelMBeansStorage The Model MBeans storage control
	 */
	private static void setModelMBeansStorage(boolean modelMBeansStorage)
	{
		RequiredModelMBean.modelMBeansStorage = modelMBeansStorage;
	}

	/**
	 * This allows all the Model MBeans caching control.
	 *
	 * @param modelMBeansCaching The Model MBeans caching control
	 */
	private static void setModelMBeansCaching(boolean modelMBeansCaching)
	{
		RequiredModelMBean.modelMBeansCaching = modelMBeansCaching;
	}

	private void initializeAttributeDescriptorsField(ModelMBeanAttributeInfo[] attrs)
	{
		for(int i=0;i<attrs.length;i++)
		{
			Descriptor ads = attrs[i].getDescriptor();

			String perStr = (String)ads.getFieldValue(DescriptorSupport.PERSIST_PERIOD);

			if(perStr == null)
				ads.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ (new Date().getTime()));
			else
			{
				long persistPeriod = Long.parseLong(perStr);
				ads.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ (new Date().getTime() - persistPeriod - 1));
			}

			if(!modelMBeansCaching)
				ads.setField(DescriptorSupport.CURRENCY_TIME_LIMIT, ""+ 0);
		}
	}

    private synchronized  void OnUpdate(Descriptor attributeDescriptor)
													throws Exception
	{
		//check if its for the first time
		boolean alreadyPersisted=findPersistent();

		//if persistent data already present
		if(alreadyPersisted)
		{
			log.trace("//readDataIntoCache");
			readDataIntoCache();

			log.trace("//removeUpdatedValueFromCache");
			updateCache(attributeDescriptor);

			log.trace("//write Data Back into file");
			writeBackIntoFile(attributeDescriptor);
		}
		else
		{
			//else
			log.trace("//write Data into file");
			writeIntoFile(attributeDescriptor);
		}
    }//OnUpdate

	private boolean findPersistent()
	{
	    FileInputStream f=null;

		try
		{
			f = new  FileInputStream(persistLocation+File.separator+persistName);
			return true;
		}
		catch(Exception e)
		{
			return false;
		}
		finally
		{
		    try
			{
				if(f!=null)f.close();
		    }
			catch(Exception ee)
			{
				log.error("Couldnt Close the File",ee);
		    }
		}
	}

	//readDataIntoCache
	private void  readDataIntoCache() throws Exception
	{
	    try
		{
			InputStream istr = new  FileInputStream(persistLocation+File.separator+persistName);
			ObjectInputStream data = new ObjectInputStream(istr);

			//chumma read the mbean info
			log.trace("Read MBeanInfo");
			data.readObject();

			nameVec = new Vector();
			valueVec = new Vector();

			while(true)
			{
				try
				{
					String s=(String)data.readObject();
					log.debug("add "+s+"to Name Vector");
					nameVec.addElement(s);
					Object o=data.readObject();
					log.debug("add "+o+ " to Value Vector");
					valueVec.addElement(o);

				}
				catch(Exception e1)
				{
					break;
				}
			}//read from old file over..

			log.debug("read from file over");
			data.close();
			istr.close();
	    }
		catch(Exception e)
		{
			log.error(e.getMessage(),e);
			throw e;
	    }
	}

	//update Cache
	private void updateCache(Descriptor attributeDescriptor) throws Exception
	{
	    try
		{
			String name  = (String)attributeDescriptor.getFieldValue(DescriptorSupport.NAME);
			Object value = getAttribute(name, false);

			log.debug("Size="+nameVec.size()+":"+valueVec.size());
			if(nameVec != null)
			{
				for(int j=0;j<nameVec.size();j++)
				{
					if(((String)nameVec.elementAt(j)).equals(name))
					{
						log.debug("Gonna Remove"+valueVec.get(j));
						nameVec.remove(j);
						valueVec.remove(j);
						break;
					}
				}

				log.debug("adding "+name+" and "+value+ "to Vector");
				nameVec .addElement(name);
				valueVec.addElement(value);
				log.debug("Size="+nameVec.size()+":"+valueVec.size());
			}
	    }
		catch(Exception e)
		{
			log.error(e.getMessage(),e);
			throw e;
	    }
	}

	//write back to file
	private void writeBackIntoFile(Descriptor attributeDescriptor ) throws Exception
	{
	    try
		{
			OutputStream ostr= new FileOutputStream(persistLocation+File.separator+persistName);
			ObjectOutputStream data = new ObjectOutputStream(ostr);

			data.writeObject(mbeanInfo);

			if(nameVec != null)
			{
				for(int i=0;i<nameVec.size();i++)
				{
					log.debug("writeObject "+nameVec.elementAt(i)+"into file");
					data.writeObject(nameVec.elementAt(i));
					log.debug("writeObject "+valueVec.elementAt(i)+"into file");
					data.writeObject(valueVec.elementAt(i));
				}
			}

			data.close();
			ostr.close();
			log.debug("SET LAST UPDATED TIME");
			attributeDescriptor.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP, ""+ (new Date().getTime()));
	    }
		catch(Exception e )
		{
			log.error(e.getMessage(),e);
			throw e;
	    }
	}

	private void writeIntoFile(Descriptor attributeDescriptor) throws Exception
	{
	    try
		{
			String name =(String)attributeDescriptor.getFieldValue(DescriptorSupport.NAME) ;
			Object value= getAttribute(name,false);

			if(persistLocation != null)
			{
				File dir =new File(persistLocation);
				if(!dir.exists())
				{
					dir.mkdir();
				}

				FileOutputStream out=new FileOutputStream(dir+File.separator+persistName);
				ObjectOutputStream data=new ObjectOutputStream(out);

				data.writeObject(mbeanInfo);
				data.writeObject(name);
				data.writeObject(value);

				data.close();
				out.close();
			}
			//Set Last Updated and Last Persisted Time

			attributeDescriptor.setField(DescriptorSupport.LAST_UPDATED_TIMESTAMP,"" + (new Date().getTime()));
			//attributeDescriptor.setField(DescriptorSupport.LAST_PERSISTED_TIMESTAMP,""+(new Date().getTime()));
	    }
		catch(Exception e)
		{
			log.error(e.getMessage(),e);
			throw e;
	    }
	}

	private ModelMBeanInfo createDefaultModelMBeanInfo()
	{
		Constructor[] cons = this.getClass().getConstructors();
		ModelMBeanConstructorInfo[] cinfo = new ModelMBeanConstructorInfo[cons.length];

		for(int i=0;i<cons.length;i++)
		{
			cinfo[i] = new ModelMBeanConstructorInfo("Constructors",cons[i]);
		}

		return (new ModelMBeanInfoSupport((this.getClass().getName()),"Default ModelMBean",null,cinfo,null,null));
	}

	/**
	 * returns the object of type as per given in the parameter string for the
	 * given value.
	 */
	private Object returnDefaultValue(Object objvalue, String type)
													throws Exception
	{
		if(type.indexOf("TabularData") != -1)
			return objvalue;

		String value = objvalue.toString();
		if(type.indexOf("String") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new String[]{value});
			else
				return value;
		}

		if(type.indexOf("Integer") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Integer[]{(new Integer(value))});
			else
				return new Integer(value);
		}
		if(type.indexOf("Long") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Long[]{(new Long(value))});
			else
				return new Long(value);
		}
		if(type.indexOf("Boolean") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Boolean[]{(new Boolean(value))});
			else
				return new Boolean(value);
		}
		if(type.indexOf("Float") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Float[]{(new Float(value))});
			else
				return new Float(value);
		}
		if(type.indexOf("Double") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Double[]{(new Double(value))});
			else
				return new Double(value);
		}
		if(type.indexOf("Byte") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Byte[]{(new Byte(value))});
			else
				return new Byte(value);
		}
		if(type.indexOf("Character") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Character[]{(new Character(value.charAt(0)))});
			else
				return new Character(value.charAt(0));
		}
		if(type.indexOf("Short") != -1)
		{
			if(type.startsWith("[L") && type.endsWith(";"))
				return (Object)(new Short[]{(new Short(value))});
			else
				return new Short(value);
		}
		//if(type.indexOf("TabularData"))
		//	return new TabularData(value);
		//if(type.indexOf("CompositeData"))
		//	return new CompositeData(value);

		//throw new Exception("Composite/Tabular Data cannot be formed");
		return null;
	}

	private Object invokeFriend(String actionName, Object[] params,
								String[] signature, Object source)
									throws MBeanException, ReflectionException
	{
		try
		{

			Class e = source.getClass();

			if(signature == null)
				signature = new String[0];

			Class parameterTypes[]=new Class[signature.length];
			for(int i=0; i<signature.length; i++)
				//parameterTypes[i] = Class.forName(signature[i]);
			    parameterTypes[i] = Thread.currentThread().getContextClassLoader().loadClass(signature[i]);

			Method attrMethod = e.getMethod(actionName,parameterTypes);

			return attrMethod.invoke(source,params);

		}
		catch(Exception e)
		{
			if(e instanceof InvocationTargetException)
			{
				Exception e1 = (Exception)((InvocationTargetException)e).getTargetException();
				throw new ReflectionException(e1);
			}

			throw new ReflectionException(e);
		}
	}

	private Object invoke(String actionName, Object[] params, String[] signature, boolean flag)
		//throws Exception
		throws javax.management.MBeanException,
				javax.management.ReflectionException
	{
		try
		{
			Object source = null;

			if(flag)
				source = this;
			else
				source = object;

			Class e = source.getClass();

			if(signature == null)
			    // throw new MBeanException(new ServiceNotFoundException("Could Not Invoke the operation"+actionName));
			    signature = new String[0];

			Class parameterTypes[]=new Class[signature.length];
			for(int i=0; i<signature.length; i++)
			{
				//parameterTypes[i] = params[i].getClass();
				try
				{
					parameterTypes[i] =  Thread.currentThread().getContextClassLoader().loadClass(signature[i]);
				}
				catch(Exception ee)
				{
					parameterTypes[i] = getProperClass(signature[i]);
					if (parameterTypes[i] == null && params[i] != null)
					{
						parameterTypes[i] = params[i].getClass().getClassLoader().loadClass(signature[i]);
					}
				}

			}

			Method attrMethod = e.getMethod(actionName,parameterTypes);
			return attrMethod.invoke(source,params);
		}
		catch(Exception e)
		{
			//e.printStackTrace();
			if(e instanceof InvocationTargetException)
			{
				Exception e1 = (Exception)((InvocationTargetException)e).getTargetException();
				throw new ReflectionException(e1);
			}

			throw new ReflectionException(e);
		}
	}

	private void restoreTabularData(Object value, Hashtable tempTable)
	{
		if(value instanceof TabularData)
		{
			TabularData data = (TabularData)value;
			for(Enumeration en = tempTable.keys();en.hasMoreElements();)
			{
				Object[] index = (Object[])en.nextElement();
				Object indexValue = tempTable.get(index);
				if(indexValue instanceof CompositeData)
				{
					data.addRow(index, (CompositeData)indexValue);
				}
				else if(indexValue instanceof Hashtable)
				{
					CompositeData comp = data.getRow(index);
					Hashtable ttt = (Hashtable)indexValue;
					for(Enumeration ex = ttt.keys();ex.hasMoreElements();)
					{
						String key = (String) ex.nextElement();
						comp.setDataItem(key, ttt.get(key));
					}
				}
			}
		}
	}

	private boolean isInArray(String[] indexNames, String key)
	{
		for(int i=0;i<indexNames.length;i++)
			if(indexNames[i].equals(key))
				return true;

		return false;
	}

	private MBeanNotificationInfo getNotifInfo(Notification notif)
	{
		MBeanNotificationInfo[] notifinfo = getNotificationInfo();

		for(int i=0;i<notifinfo.length;i++)
		{
			String[] notifTypes = notifinfo[i].getNotifTypes();
			for(int j=0;j<notifTypes.length;j++)
				if(notifTypes[j].equals(notif.getType()))
					return notifinfo[i];
		}

		return null;
	}

	/** Create Logger*/
	private void createLogger()
	{
		try
		{
			LogFactory.setLoggingLevel(com.adventnet.agent.logging.Level.TRACE);
			log = LogFactory.getInstance("JMX");
			log.setLoggingLevel(com.adventnet.agent.logging.Level.TRACE);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

    /**
     * This method returns the appropriate Wrapper Class
     * if the string value of the primitive data type is given.
     * @param type The String representation of the primitive data type
     * @return The appropriate wrapper class.
     */
    private static Class getProperClass(String type)
    {
        if(type.endsWith("int[]") || type.endsWith("[I"))
            return (new int[0]).getClass();

        else if(type.endsWith("long[]") || type.endsWith("[J"))
            return (new long[0]).getClass();
        else if(type.endsWith("byte[]") || type.endsWith("[B"))
            return (new byte[0]).getClass();
        else if(type.endsWith("float[]") || type.endsWith("[F"))
            return (new float[0]).getClass();
        else if(type.endsWith("char[]") || type.endsWith("[C"))
            return (new char[0]).getClass();
        else if(type.endsWith("short[]") || type.endsWith("[S"))
            return (new short[0]).getClass();
        else if(type.endsWith("double[]") || type.endsWith("[D"))
            return (new double[0]).getClass();
        else if(type.endsWith("boolean[]") || type.endsWith("[Z"))
            return (new boolean[0]).getClass();

        if(type.endsWith("int"))
            return int.class;
        else if(type.endsWith("long"))
            return long.class;
        else if(type.endsWith("byte"))
            return byte.class;
        else if(type.endsWith("float"))
            return float.class;
        else if(type.endsWith("char"))
            return char.class;
        else if(type.endsWith("short"))
            return short.class;
        else if(type.endsWith("double"))
            return double.class;
        else if(type.endsWith("boolean"))
            return boolean.class;

        return null;
    }

	private void logTheNotification(Notification ntfyObj,
									MBeanNotificationInfo notifinfo)
	{
		Descriptor descr = ((ModelMBeanNotificationInfo)notifinfo).getDescriptor();

		if((descr.getFieldValue(DescriptorSupport.LOG) != null)  &&
                   !((String)descr.getFieldValue(DescriptorSupport.LOG)).toLowerCase().equals("t"))
			return;

		String notifFileName = (String)descr.getFieldValue(DescriptorSupport.LOG_FILE);

		try
		{
			if(prints == null && (notifFileName != null))
			{
                FileOutputStream fileos = new FileOutputStream(notifFileName, true);
                prints = new PrintStream(fileos);
			}
		}
		catch(Exception e)
		{
			//e.printStackTrace();
		}

		if(prints != null)
		{
			prints.println("*** NOTIFICATION ***");
			prints.println("time     = "+new Date(ntfyObj.getTimeStamp()).toString());
			prints.println("type     = "+ntfyObj.getType());
			prints.println("message  = "+ntfyObj.getMessage());

			if(ntfyObj instanceof AttributeChangeNotification)
			{
				prints.println("oldvalue = "+((AttributeChangeNotification)ntfyObj).getOldValue());
				prints.println("newvalue = "+((AttributeChangeNotification)ntfyObj).getNewValue());
			}

			prints.println("");
		}
	}
}