/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.monitor;


/**
 * This interface exposes the remote management interface of the counter monitor MBean.
 */
public abstract interface CounterMonitorMBean extends MonitorMBean
{
	/**
	 * This method gets the value of the derived gauge.
	 *
	 * @return An instance of java.lang.Number giving the value of the derived gauge.
	 */
	public Number getDerivedGauge();

	/**
	 * This method gets the value of the derived gauge time stamp.It is the
	 * time when the notification is triggered nearest to the milliseconds.
	 *
	 * @return The value of the derived gauge time Stamp.
	 */
	public long getDerivedGaugeTimeStamp();

	/**
	 * This method gets the difference mode flag value.
	 *
	 * @return true if the flag is on , false otherwise.
	 */
	public boolean getDifferenceMode();

	/**
	 * This method sets the difference mode flag value.
	 *
	 * @param true if the difference mode is used, false otherwise.
	 */
	public void setDifferenceMode(boolean value);

	/**
	 * This method gets the modulus value.
	 *
	 * @return An instance of java.lang.Number giving the modulus value.
	 */
	public Number getModulus();

	/**
	 * This method sets the modulus value .
	 *
	 * @param value An instance of java.lang.Number which is the modulus value.
	 *
	 * @exception java.lang.IllegalArgumentException - The specified modulus is
	 * 				null or the modulus value is less than zero.
	 */
	public void setModulus(Number value) throws IllegalArgumentException;

	/**
	 * This method gets the notification's on/off switch value.
	 *
	 * @return true if the counter monitor notifies when exceeding the
	 * 				threshold, false otherwise.
	 */
	public boolean getNotify();

	/**
	 * This method sets the notification's on/off switch value.
	 *
	 * @param value The notification's on/off switch value.
	 */
	public void setNotify(boolean value);

	/**
	 * This method gets offset value .
	 *
	 * @return An instance of java.lang.Number giving the offset value .
	 */
	public Number getOffset();

	/**
	 * This method sets the offset value .
	 *
	 * @param value The offset value.
	 */
	public void setOffset(Number value) throws IllegalArgumentException;

	/**
	 * This method gets the threshold value .
	 *
	 * @return An instance of java.lang.Number giving the threshold .
	 */
	public Number getThreshold();

	/**
	 * This method sets the threshold value .
	 *
	 * @param value The Threshold Value.
	 *
	 * @exception java.lang.IllegalArgumentException - The specified threshold
	 * 				is null or the threshold value is less than zero.
	 */
	public void setThreshold(Number value) throws IllegalArgumentException;
}