/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.monitor;


/**
 * Exposes the remote management interface of the gauge monitor MBean.
 */
public abstract interface GaugeMonitorMBean extends MonitorMBean
{
	/**
	 * This method gets the value of the derived gauge. The derived gauge is
	 * either the exact value of the observed attribute , or the difference
	 * between the two consecutive observed values of the attribute.
	 *
	 * @return An instance of java.lang.Number giving the value of the derived gauge.
	 */

	public Number getDerivedGauge();

	/**
	 * This method gets the value of the derived gauge time stamp.The derived
	 * gauge time stamp is the value(in the nearest miliseconds) when the
	 * notification was triggered.
	 *
	 * @return long value representing the time the notification was triggered.
	 */
	public long getDerivedGaugeTimeStamp();

	/**
	 * This method gets the value of the difference mode. If the difference
	 * mode is true, the difference mode option is set to calculate the value
	 * of the derived gauge.
	 *
	 * @return boolean value indicating whether the difference mode option is set.
	 */
	public boolean getDifferenceMode();

	/**
	 * This method sets the state of the difference mode.
	 *
	 * @param value boolean value representing the state of the difference mode.
	 */
	public void setDifferenceMode(boolean value);

	/**
	 * This method gets the value of the high threshold.
	 *
	 * @return An instance of java.lang.Number giving the value of the high threshold.
	 */
	public Number getHighThreshold();

	/**
	 * This method gets the value of the low threshold.
	 *
	 * @return An instance of java.lang.Number giving the value of the low threshold.
	 */
	public Number getLowThreshold();

	/**
	 * This method sets the high threshold and low threshold values.
	 *
	 * @param	highValue - The high threshold value.
	 *
	 * @param	lowValue - The low threshold value.
	 *
	 * @exception java.lang.IllegalArgumentException - The specified high/low
	 * 				threshold is null or the low threshold is greater than the
	 * 				high threshold or the high threshold and the low threshold
	 * 				are not of the same type.
	 */
	public void setThresholds(Number highvalue, Number lowValue)
		throws IllegalArgumentException;

	/**
	 * This method gets the high notification's on/off switch value.
	 *
	 * @return true if the gauge monitor notifies when exceeding the high
	 * 				threshold, false otherwise.
	 */
	public boolean getNotifyHigh();

	/**
	 * This method sets the high notification's on/off switch value.
	 *
	 * @param value - The high notification's on/off switch value.
	 */
	public void setNotifyHigh(boolean value);

	/**
	 * This method gets the low notification's on/off switch value.
	 *
	 * @return true if the gauge monitor notifies when exceeding the low
	 * 				threshold, false otherwise.
	 */
	public boolean getNotifyLow();

	/**
	 * This method sets the low notification's on/off switch value.
	 *
	 * @param value - The low notification's on/off switch value.
	 */
	public void setNotifyLow(boolean value);
}