/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.monitor;

import javax.management.ObjectName;

/**
 * This interface exposes the remote management interface of monitor MBeans.
 */
public abstract interface MonitorMBean
{
	/**
	 * This method gets the granularity period (in milliseconds).
	 *
	 * @return long value representing the value of the granularity period
	 * 				(in milliseconds).
	 */
	public long getGranularityPeriod();

	/**
	 * This method sets the granularity period (in milliseconds).
	 *
	 * @param period the granularity period value.
	 *
	 * @exception java.lang.IllegalArgumentException - The granularity period
	 * 				is less than or equal to zero.
	 */
	public void setGranularityPeriod(long period) throws IllegalArgumentException;

	/**
	 * This method gets the name of the attribute being observed.
	 *
	 * @return String The name of the attribute that is being observed.
	 */
	public String getObservedAttribute();

	/**
	 * This method sets the attribute being observed.
	 *
	 * @param attribute The attribute to be observed.
	 */
	public void setObservedAttribute(String attribute);

	/**
	 * This method gets the object name of the object being observed.
	 *
	 * @return The ObjectName of the object being observed.
	 */
	public ObjectName getObservedObject();

	/**
	 * This method sets the object name of the object being observed.
	 *
	 * @param object The ObjectName of the object to be observed.
	 */
	public void setObservedObject(ObjectName object);

	/**
	 * This method tests whether the monitor MBean is active. A monitor MBean
	 * is marked active when the start method is called. It becomes inactive
	 * when the stop method is called.
	 *
	 * @return boolean value indicating whether the MBean is active or not.
	 */
	public boolean isActive();

	/**
	 * This method starts the monitor.
	 */
	public void start();

	/**
	 * This method stops the monitor.
	 */
	public void stop();
}