/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.io.Serializable;

/**
 * Represents the result of a multiple access to several roles of a relation
 * (either for reading or writing).
 */
public class RoleResult implements Serializable
{
    // List of roles successfully accessed
	RoleList roleList = null;

    // List of roles unsuccessfully accessed
	RoleUnresolvedList unresolvedRoleList = null;

	/**
	 * Constructor to create RoleResult
	 *
	 * @param theRoleList - list of roles succesfully accessed
	 *
	 * @param theRoleUnresList - list of roles not accessed (with problem descriptions)
	 */
	public RoleResult(RoleList theRoleList, RoleUnresolvedList theRoleUnresList)
	{
		this.roleList = theRoleList;
		this.unresolvedRoleList = theRoleUnresList;
    }

	/**
	 * Retrieves list of roles successfully accessed
	 *
	 * @return a RoleList
	 */
	public RoleList getRoles()
	{
		return this.roleList;
	}

	/**
	 * Retrieves list of roles unsuccessfully accessed
	 *
	 * @return a RoleUnresolvedList
	 */
	public RoleUnresolvedList getRolesUnresolved()
	{
		return this.unresolvedRoleList;
	}

	/**
	 * Sets list of roles successfully accessed
	 *
	 * @param theRoleList - list of roles successfully accessed
	 */
	public void setRoles(RoleList theRoleList)
	{
		this.roleList = theRoleList;
	}

	/**
	 * Sets list of roles unsuccessfully accessed
	 *
	 * @param theRoleUnresolvedList - list of roles unsuccessfully accessed
	 */
	public void setRolesUnresolved(RoleUnresolvedList theRoleUnresList)
	{
		this.unresolvedRoleList = theRoleUnresList;
	}
}