/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * A RoleUnresolvedList represents a list of RoleUnresolved objects,
 * representing roles not retrieved from a relation due to a problem
 * encountered whe trying to access (read or write to roles)
 */
public class RoleUnresolvedList extends ArrayList
{
	/**
	 * Constructs an empty RoleUnresolvedList.
     */
	public RoleUnresolvedList()
	{
		super();
	}

	/**
	 * Constructs an empty RoleUnresolvedList with the initial capacity specified.
	 *
	 * @param theInitialCapacity - initial capacity
	 */
	public RoleUnresolvedList(int theInitialCapacity)
	{
		super(theInitialCapacity);
	}

	/**
	 * Constructs a RoleUnresolvedList containing the elements of the ArrayList
	 * specified, in the order in which they are returned by the ArrayList's
	 * iterator. The RoleUnresolvedList instance has an initial capacity
	 * of 110% of the size of the ArrayList specified.
	 *
	 * @param theList - list of RoleUnresolved objects
	 *
	 * @exception java.lang.IllegalArgumentException - if:
	 * 				- null parameter or
	 *           	- an element in the ArrayList is not a RoleUnresolved
	 */
	public RoleUnresolvedList(List theList) throws IllegalArgumentException
	{
		super(theList);

		if(theList == null)
			throw new IllegalArgumentException();

		Iterator iter = super.iterator();

		while(iter.hasNext())
		{
			Object temp = iter.next();

			if(!(temp instanceof RoleUnresolved))
				throw new IllegalArgumentException();
		}
	}

	/**
	 * Inserts the unresolved role specified as an element at the position
	 * specified. Elements with an index greater than or equal to the current
	 * position are shifted up.
	 *
	 * @param  theIndex - - The position in the list where the new
	 * 				RoleUnresolved object is to be inserted.
	 *
	 * @param theRoleUnresolved - - The RoleUnresolved object to be inserted.
	 *
	 * @throws java.lang.IllegalArgumentException - if the unresolved role is null.
	 *
	 * @throws java.lang.IndexOutOfBoundsException If the index exceeds the bounds
	 */
	public void add(int theIndex,RoleUnresolved theRoleUnres)
						         throws IllegalArgumentException,
										IndexOutOfBoundsException
	{
		if(theRoleUnres == null)
			throw new IllegalArgumentException();

		super.add(theIndex,theRoleUnres);
	}

	/**
	 * Adds the RoleUnresolved specified as the last element of the list.
	 *
	 * @param theRoleUnres - - the unresolved role to be added.
	 *
	 * @exception java.lang.IllegalArgumentException - if the unresolved role is null.
	 */
	public void add(RoleUnresolved theRoleUnres) throws IllegalArgumentException
	{
		if(theRoleUnres == null)
			throw new IllegalArgumentException();

		super.add(theRoleUnres);
	}

	/**
	 * Inserts all of the elements in the RoleUnresolvedList specified into
	 * this list, starting at the specified position, in the order in which
	 * they are returned by the Iterator of the RoleUnresolvedList specified.
	 *
	 * @param theIndex - - Position at which to insert the first element from
	 * 				the RoleUnresolvedList specified.
	 *
	 * @param theRoleUnresolvedList - - Elements to be inserted into the list.
	 *
	 * @throws java.lang.IllegalArgumentException - if the role is null.
	 *
	 * @throws java.lang.IndexOutOfBoundsException - if accessing with an
	 * 				index outside of the list.
	 *
	 * @return True if the operation is successful otherwise false
	 */
	public boolean addAll(int theIndex, RoleUnresolvedList theRoleUnresolvedList)
			               throws IllegalArgumentException,
								  IndexOutOfBoundsException
	{
		if(theRoleUnresolvedList == null)
			throw new IllegalArgumentException();

		return super.addAll(theIndex,theRoleUnresolvedList);
	}

	/**
	 * Appends all the elements in the RoleUnresolvedList specified to the end
	 * of the list, in the order in which they are returned by the Iterator of
	 * the RoleUnresolvedList specified.
	 *
	 * @param theRoleUnresolvedList - - Elements to be inserted into
	 * 				the list (can be null).
	 *
	 * @exception java.lang.IndexOutOfBoundsException - if accessing with
	 * 				an index outside of the list.
	 */
	public boolean addAll(RoleUnresolvedList theRoleUnresolvedList)
								throws IndexOutOfBoundsException
	{
		return super.addAll(theRoleUnresolvedList);
	}

	/**
	 * Cloning Returns a new RoleList, but no copy of the included elements.
	 *
	 * Overrides:
	 *           clone in class java.util.ArrayList
	 *
	 * @return A new RoleList, but no copy of the included elements.
	 */
	public java.lang.Object clone()
	{
		return new RoleUnresolvedList( (ArrayList) super.clone() );
	}

	/**
	 * Sets the element at the position specified to be the unresolved role
	 * specified. The previous element at that position is discarded.
	 *
	 * @param theIndex - - The position specified.
	 *
	 * @param theRoleUnres - - The value to which the unresolved role element
	 * 				should be set.
	 *
	 * @exception java.lang.IllegalArgumentException - if the unresolved role is null.
	 *
	 * @exception java.lang.IndexOutOfBoundsException if accessing with
	 * 				an index outside of the list.
	 */
	public void set(int theIndex, RoleUnresolved theRoleUnres)
				         throws IllegalArgumentException,
								IndexOutOfBoundsException
	{
		if(theRoleUnres == null)
			throw new IllegalArgumentException();

		super.set(theIndex,theRoleUnres);
	}
}