/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.timer;

import java.util.*;

/** A WorkerThread for running tasks in a Java VM */

class WorkerThread extends Thread
{

    Scheduler scheduler = null;

    // for logging
    public    int             m_current_indent_count    = 0;
    public    StringBuffer    m_current_indent_buffer   = null;

    /** The constructor may not be necessary **/
    public WorkerThread(Scheduler s,String nam)
    {
        super(nam);
        scheduler = s;
        m_current_indent_buffer = new StringBuffer();
    }

    /** Wake this one up, if waiting **/
    synchronized void wakeUp()
    {
        notifyAll();
    }

    /** Wait here until asked to to another task **/
    synchronized void waitUntilAsked()
    {
        while ((scheduler.ready_tasks.size() == 0) && (!scheduler.STOP_ALL))
        {
            try
            {
                wait(100);
            }
            catch (InterruptedException iex) {}
        }
    }

    /** The run methos does the tasks as they come in **/
    public void run()
    {
        while ( (!scheduler.STOP_ALL) && (!scheduler.STOP_THIS) )
            try
            {
                waitUntilAsked();
                Runnable task = scheduler.getNextTask();
                if (task != null) task.run();
            }
            catch (Exception ex)
            {
                System.err.println("Exception running task: "+ ex);
//                ex.printStackTrace();
            }
        scheduler.NUM_THREADS_STOPPED += 1;
        scheduler.TOTAL_THREADS -= 1;
    }

}